@@ -40,14 +40,15 @@
 #include "bugzillalib.h"
 #include "crashedapplication.h"
 #include "drkonqi.h"
+#include "drkonqi_debug.h"
 #include "drkonqi_globals.h"
+#include "reportinformationdialog.h"
 #include "reportinterface.h"
 #include "statuswidget.h"
 #include "systeminformation.h"
 
 using namespace std::chrono_literals;
 
-static const char kWalletEntryName[] = "drkonqi_bugzilla";
 static const char kWalletEntryUsername[] = "username";
 static const char kWalletEntryPassword[] = "password";
 
@@ -59,6 +60,7 @@ static const char konquerorKWalletEntryPassword[] = "Bugzilla_password";
 
 BugzillaLoginPage::BugzillaLoginPage(ReportAssistantDialog *parent)
     : ReportAssistantPage(parent)
+    , m_walletEntryName(DrKonqi::isTestingBugzilla() ? QStringLiteral("drkonqi_bugzilla_test_mode") : QStringLiteral("drkonqi_bugzilla"))
     , m_wallet(nullptr)
     , m_walletWasOpenedBefore(false)
 {
@@ -172,6 +174,7 @@ void BugzillaLoginPage::openWallet()
 {
     // Store if the wallet was previously opened so we can know if we should close it later
     m_walletWasOpenedBefore = KWallet::Wallet::isOpen(KWallet::Wallet::NetworkWallet());
+    qCDebug(DRKONQI_LOG) << "Wallet was open?" << m_walletWasOpenedBefore;
 
     // Request open the wallet
     WId windowId = 0;
@@ -182,12 +185,15 @@ void BugzillaLoginPage::openWallet()
     }
 
     m_wallet = KWallet::Wallet::openWallet(KWallet::Wallet::NetworkWallet(), windowId);
+    qCDebug(DRKONQI_LOG) << "wallet?" << m_wallet;
 }
 
 void BugzillaLoginPage::walletLogin()
 {
     if (!m_wallet) {
-        if (kWalletEntryExists(QLatin1String(kWalletEntryName))) { // Key exists!
+        qCDebug(DRKONQI_LOG) << "Wallet not open";
+        if (kWalletEntryExists(m_walletEntryName)) { // Key exists!
+            qCDebug(DRKONQI_LOG) << "wallet entry exists";
             openWallet();
             ui.m_savePasswordCheckBox->setCheckState(Qt::Checked);
             // Was the wallet opened?
@@ -196,7 +202,7 @@ void BugzillaLoginPage::walletLogin()
 
                 // Use wallet data to try login
                 QMap<QString, QString> values;
-                m_wallet->readMap(QLatin1String(kWalletEntryName), values);
+                m_wallet->readMap(m_walletEntryName, values);
                 QString username = values.value(QLatin1String(kWalletEntryUsername));
                 QString password = values.value(QLatin1String(kWalletEntryPassword));
 
@@ -206,6 +212,7 @@ void BugzillaLoginPage::walletLogin()
                 }
             }
         } else if (kWalletEntryExists(konquerorKWalletEntryName)) {
+            qCDebug(DRKONQI_LOG) << "wallet entry does not exist";
             // If the DrKonqi entry is empty, but a Konqueror entry exists, use and copy it.
             openWallet();
             if (m_wallet) {
@@ -222,7 +229,7 @@ void BugzillaLoginPage::walletLogin()
                     values.clear();
                     values.insert(QLatin1String(kWalletEntryUsername), username);
                     values.insert(QLatin1String(kWalletEntryPassword), password);
-                    m_wallet->writeMap(QLatin1String(kWalletEntryName), values);
+                    m_wallet->writeMap(m_walletEntryName, values);
 
                     ui.m_savePasswordCheckBox->setCheckState(Qt::Checked);
 
@@ -248,9 +255,11 @@ void BugzillaLoginPage::loginClicked()
     updateWidget(false);
 
     if (ui.m_savePasswordCheckBox->checkState() == Qt::Checked) { // Wants to save data
+        qCDebug(DRKONQI_LOG) << "trying to store credentials";
         if (!m_wallet) {
             openWallet();
         }
+        qCDebug(DRKONQI_LOG) << "Wallet opened?" << m_wallet;
         // Got wallet open ?
         if (m_wallet) {
             m_wallet->setFolder(KWallet::Wallet::FormDataFolder());
@@ -258,17 +267,17 @@ void BugzillaLoginPage::loginClicked()
             QMap<QString, QString> values;
             values.insert(QLatin1String(kWalletEntryUsername), ui.m_userEdit->text());
             values.insert(QLatin1String(kWalletEntryPassword), ui.m_passwordEdit->password());
-            m_wallet->writeMap(QLatin1String(kWalletEntryName), values);
+            m_wallet->writeMap(m_walletEntryName, values);
         }
     } else { // User doesn't want to save or wants to remove.
-        if (kWalletEntryExists(QLatin1String(kWalletEntryName))) {
+        if (kWalletEntryExists(m_walletEntryName)) {
             if (!m_wallet) {
                 openWallet();
             }
             // Got wallet open ?
             if (m_wallet) {
                 m_wallet->setFolder(KWallet::Wallet::FormDataFolder());
-                m_wallet->removeEntry(QLatin1String(kWalletEntryName));
+                m_wallet->removeEntry(m_walletEntryName);
             }
         }
     }
@@ -370,6 +379,33 @@ BugzillaInformationPage::BugzillaInformationPage(ReportAssistantDialog *parent)
     loadDistroCombo();
 }
 
+// Helps to track textual blocks we insert into text edits.
+// This is intended to ensure the cursor is at the end of the first block
+// e.g.
+//   foo:\n
+//    <- cursor here
+//   bar:\n
+// To that end the function accepts an input position which is -1 when not initialized.
+// The first block initializes it to the position after appending, all subsquent blocks
+// return the input position again. Blocks prepend themselves when the position is >=0
+// Nice to have and easy to do but not strictly necessary -> https://bugs.kde.org/show_bug.cgi?id=438736
+static int appendAndKeepPosition(QTextEdit *edit, const QString &text, int position)
+{
+    const bool hasPosition = position >= 0;
+
+    if (hasPosition) {
+        edit->append(QString()); // append empty paragraph in case there was previous block (i.e. add extra newline)
+    }
+
+    edit->append(text); // modifies cursor
+
+    if (!hasPosition) {
+        return edit->textCursor().position();
+    }
+
+    return position;
+}
+
 void BugzillaInformationPage::aboutToShow()
 {
     // Calculate the minimum number of characters required for a description
@@ -379,18 +415,22 @@ void BugzillaInformationPage::aboutToShow()
     m_requiredCharacters = 20 + (reportInterface()->selectedOptionsRating() * multiplier);
 
     // Fill the description textedit with some headings:
-    QString descriptionTemplate;
     if (ui.m_detailsEdit->toPlainText().isEmpty()) {
+        int position = -1;
         if (reportInterface()->userCanProvideActionsAppDesktop()) {
-            descriptionTemplate += QLatin1String("- What I was doing when the application crashed:\n\n");
+            position = appendAndKeepPosition(ui.m_detailsEdit, QStringLiteral("- What I was doing when the application crashed:\n"), position);
         }
         if (reportInterface()->userCanProvideUnusualBehavior()) {
-            descriptionTemplate += QLatin1String("- Unusual behavior I noticed:\n\n");
+            position = appendAndKeepPosition(ui.m_detailsEdit, QStringLiteral("- Unusual behavior I noticed:\n"), position);
         }
         if (reportInterface()->userCanProvideApplicationConfigDetails()) {
-            descriptionTemplate += QLatin1String("- Custom settings of the application:\n\n");
+            position = appendAndKeepPosition(ui.m_detailsEdit, QStringLiteral("- Custom settings of the application:\n"), position);
+        }
+        if (position >= 0) {
+            QTextCursor cursor = ui.m_detailsEdit->textCursor();
+            cursor.setPosition(position);
+            ui.m_detailsEdit->setTextCursor(cursor);
         }
-        ui.m_detailsEdit->setText(descriptionTemplate);
     }
     // If attaching this report to an existing one then the title is not needed
     bool showTitle = (reportInterface()->attachToBugNumber() == 0);
@@ -453,7 +493,7 @@ void BugzillaInformationPage::loadDistroCombo()
 
     Bugzilla::BugFieldClient client;
     auto job = client.getField(QStringLiteral("rep_platform"));
-    connect(job, &KJob::finished, this, [this, &client](KJob *job) {
+    connect(job, &KJob::finished, this, [this, client](KJob *job) {
         try {
             Bugzilla::BugField::Ptr field = client.getField(job);
             if (!field) {
@@ -472,7 +512,8 @@ void BugzillaInformationPage::loadDistroCombo()
 void BugzillaInformationPage::setDistros(const Bugzilla::BugField::Ptr &field)
 {
     ui.m_distroChooserCombo->clear();
-    for (const auto &distro : field->values()) {
+    const QList<Bugzilla::BugFieldValue *> distros = field->values();
+    for (const auto &distro : distros) {
         ui.m_distroChooserCombo->addItem(distro->name());
     }
 
@@ -582,7 +623,7 @@ void BugzillaInformationPage::aboutToHide()
 
     if (ui.m_distroChooserCombo->isVisible()) {
         // Save bugzilla platform (distribution)
-        QString bugzillaPlatform = ui.m_distroChooserCombo->itemData(ui.m_distroChooserCombo->currentIndex()).toString();
+        const QString bugzillaPlatform = ui.m_distroChooserCombo->itemText(ui.m_distroChooserCombo->currentIndex());
         KConfigGroup config(KSharedConfig::openConfig(), "BugzillaInformationPage");
         config.writeEntry("BugzillaPlatform", bugzillaPlatform);
         DrKonqi::systemInformation()->setBugzillaPlatform(bugzillaPlatform);
